@@ -16,7 +16,6 @@ import json
 from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 
-
 from veracode_api_signing.exceptions import VeracodeAPISigningException
 from veracode_api_signing.plugin_requests import RequestsAuthPluginVeracodeHMAC
 
@@ -116,14 +115,14 @@ class VeracodeAPI:
         else:
             return ""
 
-    def _rest_paged_request(self, url, method, element, params=None):
+    def _rest_paged_request(self, uri, method, element, params=None):
         all_data = []
         page = 0
         more_pages = True
 
         while more_pages:
             params['page']=page
-            page_data = self._rest_request(url,method,params)
+            page_data = self._rest_request(uri,method,params)
             total_pages = page_data.get('page', {}).get('total_pages', 0)
             data_page = page_data.get('_embedded', {}).get(element, [])
             all_data += data_page  
@@ -202,6 +201,32 @@ class VeracodeAPI:
 
         return self._rest_request(uri,"GET")
 
+    def get_app_by_name (self,appname):
+        """Gets a list of applications having a name that matches appname, using the Veracode Applications API."""
+        params = {"name": appname}
+        return self._rest_paged_request(uri="appsec/v1/applications",method="GET",element="applications",params=params)
+
+    def create_app(self,app_name,business_criticality, business_unit=None, teams=[]):
+        app_def = {'name':app_name, 'business_criticality':business_criticality}
+
+        if len(teams) > 0:
+            # optionally pass a list of teams to add to the application profile
+            team_list = []
+            for team in teams:
+                team_list.append({'guid': team})
+            app_def.update({'teams': team_list})
+
+        if business_unit != None:
+            bu = {'business_unit': {'guid': business_unit}}
+            app_def.update(bu)
+
+        payload = json.dumps({"profile": app_def})
+        return self._rest_request('appsec/v1/applications','POST',body=payload)
+
+    def delete_app (self,guid):
+        uri = 'appsec/v1/applications/{}'.format(guid)
+        return self._rest_request(uri,"DELETE")
+
     def get_policy (self,guid):
         policy_base_uri = "appsec/v1/policies/{}"
         uri = policy_base_uri.format(guid)
@@ -244,6 +269,36 @@ class VeracodeAPI:
     def get_creds (self):
         return self._rest_request("api/authn/v2/api_credentials","GET")
 
+    def create_user (self,email,firstname,lastname,username=None,type="HUMAN",roles=[],teams=[]):
+        user_def = { "email_address": email, "first_name": firstname, "last_name": lastname }
+
+        rolelist = []
+        if len(roles) > 0:
+            for role in roles:
+                rolelist.append({"role_name": role})
+            user_def.append({"roles":rolelist})
+
+        if type == "API":
+            user_def.update({"user_name": username})
+            user_def.update({"permissions": [{"permission_name": "apiUser"}]})
+            if len(roles) == 0:
+               rolelist.append({"role_name": "uploadapi"})
+               rolelist.append({"role_name":"apisubmitanyscan"})
+        else:
+            if len(roles) == 0:
+               rolelist.append({"role_name":"submitter"}) 
+
+        teamlist = []
+        if len(teams) > 0:
+            for team in teams:
+                teamlist.append({"team_id": team})
+            user_def.update({"teams": teamlist})
+
+        user_def.update({"roles": rolelist})
+
+        payload = json.dumps(user_def)
+        return self._rest_request('api/authn/v2/users','POST',body=payload)
+
     def update_user (self,user_guid,roles):
         request_params = {'partial':'TRUE',"incremental": 'TRUE'}
         uri = "api/authn/v2/users/{}".format(user_guid)
@@ -255,6 +310,43 @@ class VeracodeAPI:
         payload = json.dumps({'active': False})
         return self._rest_request(uri,"PUT",request_params,payload)
 
+    def delete_user (self,user_guid):
+        uri = 'api/authn/v2/users/{}'.format(user_guid)
+        return self._rest_request(uri,"DELETE")
+
+    def get_teams (self, all_for_org=False):
+        #Gets a list of teams using the Veracode Identity API       
+        if all_for_org:
+            request_params = {'all_for_org': True}
+        else:
+            request_params = {'page': 0} #initialize the page request
+        return self._rest_paged_request("api/authn/v2/teams","GET","teams",request_params)
+
+    def create_team (self, team_name, business_unit=None, members=[]):        
+        team_def = {'team_name': team_name}
+        
+        if len(members) > 0:
+            # optionally pass a list of usernames to add as team members
+            users = []
+            for member in members:
+                users.append({'user_name': member})
+            team_def.update({'users': users})
+
+        if business_unit != None:
+            bu = {'bu_id': business_unit}
+            team_def.update(bu)
+
+        payload = json.dumps(team_def)
+        return self._rest_request('api/authn/v2/teams','POST',body=payload)
+
+    def delete_team (self, team_guid):
+        uri = 'api/authn/v2/teams/{}'.format(team_guid)
+        return self._rest_request(uri,"DELETE")
+            
+    def get_business_units (self):
+        request_params = {'page': 0}
+        return self._rest_paged_request("api/authn/v2/business_units","GET","business_units",request_params)
+
 ## SCA APIs - note must be human user to use these, not API user
 
     def get_workspaces(self):
